package com.ruoyi.system.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import com.ruoyi.common.core.domain.entity.SysDictData;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.*;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.Yxry;
import com.ruoyi.system.domain.Zlqk;
import com.ruoyi.system.mapper.SysDictDataMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.system.mapper.HscjMapper;
import com.ruoyi.system.domain.Hscj;
import com.ruoyi.system.service.IHscjService;

import javax.servlet.http.HttpServletResponse;

/**
 * 核酸采集Service业务层处理
 *
 * @author ruoyi
 * @date 2022-11-28
 */
@Service
public class HscjServiceImpl implements IHscjService {
    @Autowired
    private HscjMapper hscjMapper;
    @Autowired
    private SysDictDataMapper sysDictDataMapper;

    /**
     * 查询核酸采集
     *
     * @param id 核酸采集主键
     * @return 核酸采集
     */
    @Override
    public Hscj selectHscjById(Long id) {
        return hscjMapper.selectHscjById(id);
    }

    /**
     * 查询核酸采集列表
     *
     * @param hscj 核酸采集
     * @return 核酸采集
     */
    @Override
    @DataScope(deptAlias = "d", userAlias = "u")
    public List<Hscj> selectHscjList(Hscj hscj) {
        return hscjMapper.selectHscjList(hscj);
    }
    /**
     * 查询导出核酸采集列表
     *
     * @param hscj 核酸采集
     * @return 核酸采集
     */
    @Override
    @DataScope(deptAlias = "d", userAlias = "u")
    public List<Hscj> selectHscjListExport(Hscj hscj) {
        return hscjMapper.selectHscjListExport(hscj);
    }

    /**
     * 新增核酸采集
     *
     * @param hscj 核酸采集
     * @return 结果
     */
    @Override
    public int insertHscj(Hscj hscj) {
        String createBy = DataUtils.getValue(hscj.getArea(), hscj.getStreet(), hscj.getCommittee());
        if (createBy.equals("null"))
            throw new ServiceException("所选居住地、街道、社区不存在");
        if (CheckUtils.checkCard(hscj.getCardNo()))
            hscj.setCheckCard("正确");
        else
            throw new ServiceException("身份证格式不正确");
//        List<Hscj> list = hscjMapper.checkCard(hscj);
//        if (!list.isEmpty())
//            throw new ServiceException("该身份证号当天不能重复提交");
        hscj.setCreateBy(createBy);
        hscj.setCreateTime(DateUtils.getNowDate());
        return hscjMapper.insertHscj(hscj);
    }

    /**
     * 修改核酸采集
     *
     * @param hscj 核酸采集
     * @return 结果
     */
    @Override
    public int updateHscj(Hscj hscj) {
        String createBy = DataUtils.getValue(hscj.getArea(), hscj.getStreet(), hscj.getCommittee());
        if (createBy.equals("null"))
            throw new ServiceException("所选居住地、街道、社区不存在");
//        List<Hscj> list = hscjMapper.checkCard(hscj);
//        if (!list.isEmpty())
//            throw new ServiceException("该身份证号当天不能重复提交");
        hscj.setCreateBy(createBy);
        hscj.setUpdateTime(DateUtils.getNowDate());
        return hscjMapper.updateHscj(hscj);
    }

    /**
     * 批量删除核酸采集
     *
     * @param ids 需要删除的核酸采集主键
     * @return 结果
     */
    @Override
    public int deleteHscjByIds(Long[] ids) {
        return hscjMapper.deleteHscjByIds(ids);
    }

    /**
     * 删除核酸采集信息
     *
     * @param id 核酸采集主键
     * @return 结果
     */
    @Override
    public int deleteHscjById(Long id) {
        return hscjMapper.deleteHscjById(id);
    }

    @Override
    public void importList(List<Hscj> list, HttpServletResponse response) {
        if (StringUtils.isNull(list) || list.size() == 0) {
            throw new ServiceException("导入数据不能为空！");
        }
        List<Hscj> trueList = new ArrayList<>();
        List<Hscj> failureList = new ArrayList<>();
        for (Hscj hscj : list) {
            if (StringUtils.isEmpty(hscj.getProvince())) {
                hscj.setProvince("河北省");
            }
            if (StringUtils.isEmpty(hscj.getCity())) {
                hscj.setCity("石家庄市");
            }
            if (StringUtils.isEmpty(hscj.getRygx())) {
                hscj.setRygx("本人");
            }
            if (StringUtils.isNotEmpty(hscj.getCardNo())
                    && StringUtils.isNotEmpty(hscj.getArea())
                    && StringUtils.isNotEmpty(hscj.getBoxhao())
                    && StringUtils.isNotEmpty(hscj.getCjgh())
                    && StringUtils.isNotEmpty(hscj.getUserName())
                    && StringUtils.isNotEmpty(hscj.getSex())
                    && StringUtils.isNotEmpty(hscj.getAge().toString())
                    && StringUtils.isNotEmpty(hscj.getCjTime().toString())
                    && StringUtils.isNotEmpty(hscj.getCjName())
                    && StringUtils.isNotEmpty(hscj.getCjPhone())
                    && StringUtils.isNotEmpty(hscj.getBblx())
            ) {
                // 校验单张Excel是否有多个重复的身份证号
//                if (StringUtils.isNotEmpty(hscj.getCardNo())){
//                    int isExist = 0;
//                    for (Hscj hscjTest : list){
//                        if (hscj.getCardNo().equals(hscjTest.getCardNo())){
//                            isExist++;
//                        }
//                    }
//                    if (isExist > 1){
//                        failureList.add(hscj);
//                        continue;
//                    }
                List<Hscj> hsctList = list.stream().filter(l ->
                                hscj.getCardNo().equals(l.getCardNo()) && l.getCjTime() != null &&
                                        DateUtils.dateTime(hscj.getCjTime()).equals(DateUtils.dateTime(l.getCjTime())))
                        .collect(Collectors.toList());
                if (hsctList.size() > 1) {
                    failureList.add(hscj);
                    continue;
                }
//                }
//                // 校验四级联动是否对应字典的值
//                // 校验 所在区
//                if (StringUtils.isNotEmpty(hscj.getArea())){
//                    List<SysDictData> residential_area = DictUtils.getDictCache("residential_area");
//                    List<String> values = residential_area.stream().filter(l->hscj.getArea().equals(l.getDictLabel()))
//                            .map(SysDictData::getDictValue).collect(Collectors.toList());
//                    if (!values.isEmpty())
//                        hscj.setArea(values.get(0));
//                    else {
//                        failureList.add(hscj);
//                        continue;
//                    }
//                }
//                // 校验 街道/镇
//                if (StringUtils.isNotEmpty(hscj.getStreet())){
//                    List<SysDictData> street_town = DictUtils.getDictCache("street_town");
//                    List<String> values = street_town.stream().filter(l->hscj.getStreet().equals(l.getDictLabel()))
//                            .map(SysDictData::getDictValue).collect(Collectors.toList());
//                    if (!values.isEmpty())
//                        hscj.setStreet(values.get(0));
//                    else {
//                        failureList.add(hscj);
//                        continue;
//                    }
//                }
//                // 校验 居委会
//                if (StringUtils.isNotEmpty(hscj.getCommittee())){
//                    List<SysDictData> committee = DictUtils.getDictCache("committee");
//                    List<String> values = committee.stream().filter(l->hscj.getCommittee().equals(l.getDictLabel()))
//                            .map(SysDictData::getDictValue).collect(Collectors.toList());
//                    if (!values.isEmpty())
//                        hscj.setCommittee(values.get(0));
//                    else {
//                        failureList.add(hscj);
//                        continue;
//                    }
//                }
//                // 校验 小区/村名称
//                if (StringUtils.isNotEmpty(hscj.getCommunity())){
//                    List<SysDictData> community = DictUtils.getDictCache("community");
//                    List<String> values = community.stream().filter(l->hscj.getCommunity().equals(l.getDictLabel()))
//                            .map(SysDictData::getDictValue).collect(Collectors.toList());
//                    if (!values.isEmpty())
//                        hscj.setCommunity(values.get(0));
//                    else {
//                        failureList.add(hscj);
//                        continue;
//                    }
//                }
//                if (StringUtils.isNotEmpty(hscj.getSex())){
//                    if (hscj.getSex().equals("女")){
//                        hscj.setSex("1");
//                    }else if (hscj.getSex().equals("男")){
//                        hscj.setSex("0");
//                    }else {
//                        failureList.add(hscj);
//                        continue;
//                    }
//                }
//                if (StringUtils.isNotEmpty(hscj.getCjResult())){
//                    if (hscj.getCjResult().equals("阳性")){
//                        hscj.setCjResult("1");
//                    }else if (hscj.getCjResult().equals("阴性")){
//                        hscj.setCjResult("0");
//                    }else {
//                        failureList.add(hscj);
//                        continue;
//                    }
//                }
                //如果小区不为空，去数据字典匹配value
                if (StringUtils.isNotEmpty(hscj.getCommunity())) {
                    List<SysDictData> communitys = DictUtils.getDictCache("community");
                    List<String> values = communitys.stream().filter(l -> hscj.getCommunity().equals(l.getDictLabel()))
                            .map(SysDictData::getDictValue).collect(Collectors.toList());
                    if (!values.isEmpty())
                        hscj.setCommunity(values.get(0));
                    else {
                        failureList.add(hscj);
                        continue;
                    }
                }
                //如果委员会不为空，去数据字典匹配value
                if (StringUtils.isNotEmpty(hscj.getCommittee())) {
                    List<SysDictData> communitys = DictUtils.getDictCache("committee");
                    List<String> values = communitys.stream().filter(l -> hscj.getCommittee().equals(l.getDictLabel()))
                            .map(SysDictData::getDictValue).collect(Collectors.toList());
                    if (!values.isEmpty())
                        hscj.setCommittee(values.get(0));
                    else {
                        failureList.add(hscj);
                        continue;
                    }
                }
                //如果街道不为空，去数据字典匹配value
                if (StringUtils.isNotEmpty(hscj.getStreet())) {
                    List<SysDictData> communitys = DictUtils.getDictCache("street_town");
                    List<String> values = communitys.stream().filter(l -> hscj.getStreet().equals(l.getDictLabel()))
                            .map(SysDictData::getDictValue).collect(Collectors.toList());
                    if (!values.isEmpty())
                        hscj.setStreet(values.get(0));
                    else {
                        failureList.add(hscj);
                        continue;
                    }
                }
                if (StringUtils.isNotEmpty(hscj.getCommunity()) && StringUtils.isEmpty(hscj.getCommittee())) {
                    failureList.add(hscj);
                    continue;
                }
                if (StringUtils.isNotEmpty(hscj.getCommittee()) && StringUtils.isEmpty(hscj.getStreet())) {
                    failureList.add(hscj);
                    continue;
                }
                if (CheckUtils.checkCard(hscj.getCardNo())) {
                    hscj.setCheckCard("正确");
                    trueList.add(hscj);
                } else {
                    hscj.setCheckCard("不正确");
                    failureList.add(hscj);
                    continue;
                }
            } else {
                failureList.add(hscj);
            }
        }
        if (!trueList.isEmpty()) {
            // 循环批量添加数据
            int toIndex = 500;
            for (int i = 0; i < trueList.size(); i += 500) {
                if (i + 500 > trueList.size()) {
                    toIndex = trueList.size() - i;
                }
                List<Hscj> list1 = trueList.subList(i, i + toIndex);
                for (Hscj hscj : list1) {
                    String createBy = DataUtils.getValue(hscj.getArea(), hscj.getStreet(), hscj.getCommittee());
                    hscj.setCreateBy(createBy);
                    hscj.setCreateTime(DateUtils.getNowDate());
                }
                hscjMapper.insertBatch(list1);
            }

//            // 因为数据量过大，所以将list拆分为每个数组100条的形式
//            List<Hscj> hscjList = new ArrayList<>();
//            int listSize = trueList.size();
//            int count = listSize / 100;
//
//            for (int i = 0; i < count; i++) {
//                hscjList = trueList.subList(i*100,(i+1)*100-1);
                //1、从集合中获取到所有查询所需的条件，进行批量数据查询
//                List<Hscj> hscjs = hscjMapper.checkCardBatch(hscjList);
//                //2、循环数据进行判断是否是新增还是更新
//                List<Hscj> updateList = new ArrayList<>();
//                List<Hscj> insertList = new ArrayList<>();
//                if (!hscjs.isEmpty()) {
//                    for (Hscj hscj : hscjs) {
//                        updateList.addAll(hscjList.stream().filter(l -> hscj.getCardNo().equals(l.getCardNo())).collect(Collectors.toList()));
//                        insertList.addAll(hscjList.stream().filter(l -> !(hscj.getCardNo().equals(l.getCardNo()))).collect(Collectors.toList()));
//                    }
//                } else {
//                    insertList.addAll(hscjList);
//                }
                //3、将数据分别写入到更新数据集合和插入数据集合中
                //4、判断新增集合和插入集合是否有数据，如果有数据则执行SQL语句
//                if (!hscjList.isEmpty()) {
//                    hscjMapper.insertBatch(hscjList);
//                }
//                if (!updateList.isEmpty()) {
//                    hscjMapper.updateBatch(updateList);
//                }
//            }
            //判断是否还有剩余？如果有甚于数据，则，则将剩余数据取出，按照上面循环体中的代码逻辑执行即可
//            if (listSize % 100 != 0) {
//                hscjList = trueList.subList(count*100,listSize-1);
//                if (!hscjList.isEmpty()) {
//                    hscjMapper.insertBatch(hscjList);
//                }
//            }
//            List<Hscj> hscjs = hscjMapper.checkCard(hscj);
//            if (!hscjs.isEmpty()) {
//                hscjMapper.updateHscj(hscj);
//                continue;
//            }
//            String createBy = DataUtils.getValue(hscj.getArea(), hscj.getStreet(), hscj.getCommittee());
//            hscj.setCreateBy(createBy);
//            hscj.setCreateTime(DateUtils.getNowDate());
//            hscjMapper.insertHscj(hscj);
        }
        if (!failureList.isEmpty()) {
            ExcelUtil<Hscj> util = new ExcelUtil<Hscj>(Hscj.class);
            util.exportExcel(response, failureList, "核算采集错误数据");
        }
    }

    @Override
    public List<Hscj> checkCard(Hscj hscj) {
        return hscjMapper.checkCard(hscj);
    }

}
